#!/usr/bin/python

BASEDIR = "/home/nasroot/foscripts"
PROGRAM = "count_history_directory"

import sys
import os
import getopt

sys.path.append(os.path.join(BASEDIR, "bin/folib"))
import Exceptions
import Utility
from CommonLogger import log_debug, log_info, log_warning, log_error, log_exception

sys.path.append(os.path.join(BASEDIR, "bin"))
import Config

def usage():
    print "Usage:"
    print "    sudo maintexec %s --file-system <file system name>" % PROGRAM
    print "    sudo maintexec %s -h" % PROGRAM
    print

def count_history_directory(filesystem):
    # Check mount status
    arc_path = "/mnt/%s/.arc" % filesystem
    if not os.path.exists(arc_path):
        log_error("The specified file system may not be mounted. %s does not exist." % arc_path)
        raise Exceptions.ConfigurationError("The specified file system may not be mounted.")

    # Check if .history directory exists
    history_path = "/mnt/%s/.history" % filesystem
    if not os.path.exists(history_path):
        log_warning(".history directory on %s does not exist. It seems no migration tasks has been executed.")
        # Finish with 0 as result.
        return 0

    # Count directories in .history
    try:
        history_dirs = os.listdir(history_path)
    except EnvironmentError, e:
        log_error("An error occurred while getting a list of files/directories under .history. error=%s, path=%s" % (str(e), history_path))
        raise

    return len(history_dirs)

def main():
    # Accept SIGINT
    Utility.enable_interrupt()

    # Argument check
    try:
        short_opts = "h"
        long_opts = ["file-system="]
        opts, args = getopt.getopt(sys.argv[1:], short_opts, long_opts)
    except getopt.GetoptError, e:
        raise Exceptions.OptionError(e)

    filesystem = None

    # Parse options
    # Ignore any arguments
    for o, a in opts:
        if o == "--file-system":
            filesystem = a
        if o == "-h":
            raise Exceptions.HelpOption()

    if filesystem is None:
        raise Exceptions.OptionError("File system name must be specified.")

    count = count_history_directory(filesystem)
    print count

if __name__ == '__main__':
    try:
        main()
    except Exceptions.HelpOption:
        usage()
        sys.exit(0)
    except Exceptions.OptionError, e:
        log_error("Option error: %s" % str(e))
        sys.exit(1)
    except KeyboardInterrupt:
        log_info("Interrupted.")
        print "Interrupted."
        sys.exit(2)
    except Exceptions.ConfigurationError, e:
        log_error("Unexpected configuration: %s" % str(e))
        sys.exit(20)
    except EnvironmentError, e:
        log_exception("A system error occurred: %s", e)
        sys.exit(99)
    except Exception, e:
        log_exception("An internal error occurred: %s", e)
        sys.exit(99)
