#!/usr/bin/python

BASEDIR = "/home/nasroot/foscripts"
PROGRAM = "get_configs"

import sys
import os
import getopt

sys.path.append(os.path.join(BASEDIR, "bin/folib"))
import HDIClient
import HDIOperation
import Exceptions
from CommonLogger import log_debug, log_info, log_warning, log_error, log_exception, setup_rotation_logger

sys.path.append(os.path.join(BASEDIR, "bin"))
import Config

class ConfigType(object):
    FILESYSTEM = 0
    CIFSSHARE = 1
    NFSSHARE = 2
    MIGTASK = 3
    LFT = 4
    RESIDENT = 5
    NETWORK = 6
    HOSTNAME = 7
    UUID = 8
    LU = 9
    VG = 10
    HISTORY = 11

def usage():
    print "Usage:"
    print "    sudo maintexec get_configs --file-systems"
    print "    sudo maintexec get_configs --history-count <file system name>"
    print "    sudo maintexec get_configs --cifs-shares"
    print "    sudo maintexec get_configs --nfs-shares"
    print "    sudo maintexec get_configs --mig-tasks"
    print "    sudo maintexec get_configs --lft"
    print "    sudo maintexec get_configs --cache-resident"
    print "    sudo maintexec get_configs --network"
    print "    sudo maintexec get_configs --hostname"
    print "    sudo maintexec get_configs --system-uuid"
    print "    sudo maintexec get_configs --lu"
    print "    sudo maintexec get_configs --vg"
    print
    print "    sudo maintexec get_configs -h|--help"
    print

def get_file_systems(conn):
    ''' Get all file system information with MAPI and return the result. '''
    if conn.host == 'localhost':
        method = HDIClient.HTTPMethod.GET
        path = "/mapi/FileSystems"
        op = HDIOperation.RESTOperation(method, path)
    else:
        prog = os.path.join(BASEDIR, "int/get_configs")
        cmd = "sudo maintexec %s --file-systems" % prog
        op = HDIOperation.CommandOperation(cmd)

    log_info("Getting file system information ...")
    try:
        ret, out, err = op.execute(conn)
    except HDIOperation.ConnectionError, e:
        log_error("Failed to communicate with %s. error=%s" % (conn.host, str(e)))
        raise Exceptions.ConnectionError(e.reason, e.code)

    if conn.host == 'localhost':
        if ret != 200:
            log_error("Unexpected HTTP status code. host=%s, path=%s, method=%s, ret=%s, out=%s, err=%s" % (conn.host, path, HDIClient.HTTPMethod.tostr(method), ret, out, err))
            raise Exceptions.OperationError("Failed to get file system configuration on the node. host=%s, error=%s" % (conn.host, err), ret)
    else:
        if ret != 0:
            log_error("Command failed. host=%s, cmd=\"%s\", ret=%s, out=%s, err=%s" % (conn.host, cmd, ret, out, err))
            raise Exceptions.OperationError("Failed to get file system configuration on the node. host=%s, error=%s" % (conn.host, err), ret)

    return out.strip()

def get_history_count(conn, fsname):
    ''' Get number of history directories of a file system. '''
    op = HDIOperation.FileSystem(conn)
    try:
        count = op.get_history_count(fsname)
    except HDIOperation.ConnectionError, e:
        log_error("Failed to communicate with %s. error=%s" % (conn.host, str(e)))
        raise Exceptions.ConnectionError(e.reason, e.code)

    return count

def get_cifs_shares(conn):
    ''' Get all CIFS share information with MAPI and return the result. '''
    if conn.host == 'localhost':
        method = HDIClient.HTTPMethod.GET
        path = "/mapi/CIFSShares"
        op = HDIOperation.RESTOperation(method, path)
    else:
        prog = os.path.join(BASEDIR, "int/get_configs")
        cmd = "sudo maintexec %s --cifs-shares" % prog
        op = HDIOperation.CommandOperation(cmd)

    log_info("Getting CIFS information ...")
    try:
        ret, out, err = op.execute(conn)
    except HDIOperation.ConnectionError, e:
        log_error("Failed to communicate with %s. error=%s" % (conn.host, str(e)))
        raise Exceptions.ConnectionError(e.reason, e.code)

    if conn.host == 'localhost':
        if ret != 200:
            log_error("An unexpected HTTP status code. host=%s, path=%s, method=%s, ret=%s, out=%s, err=%s" % (conn.host, path, HDIClient.HTTPMethod.tostr(method), ret, out, err))
            raise Exceptions.OperationError("Failed to get CIFS share configuration on the node. host=%s, error=%s" % (conn.host, err), ret)
    else:
        if ret != 0:
            log_error("Command failed. host=%s, cmd=\"%s\", ret=%s, out=%s, err=%s" % (conn.host, cmd, ret, out, err))
            raise Exceptions.OperationError("Failed to get CIFS share configuration on the node. host=%s, error=%s" % (conn.host, err), ret)

    return out.strip()

def get_nfs_shares(conn):
    ''' Get all NFS share information with MAPI and return the result. '''
    if conn.host == 'localhost':
        method = HDIClient.HTTPMethod.GET
        path = "/mapi/NFSShares"
        op = HDIOperation.RESTOperation(method, path)
    else:
        prog = os.path.join(BASEDIR, "int/get_configs")
        cmd = "sudo maintexec %s --nfs-shares" % prog
        op = HDIOperation.CommandOperation(cmd)

    log_info("Getting NFS information ...")
    try:
        ret, out, err = op.execute(conn)
    except HDIOperation.ConnectionError, e:
        log_error("Failed to communicate with %s. error=%s" % (conn.host, str(e)))
        raise Exceptions.ConnectionError(e.reason, e.code)

    if conn.host == 'localhost':
        if ret != 200:
            log_error("An unexpected HTTP status code. host=%s, path=%s, method=%s, ret=%s, out=%s, err=%s" % (conn.host, path, HDIClient.HTTPMethod.tostr(method), ret, out, err))
            raise Exceptions.OperationError("Failed to get NFS share configuration on the node. host=%s, error=%s" % (conn.host, err), ret)
    else:
        if ret != 0:
            log_error("Command failed. host=%s, cmd=\"%s\", ret=%s, out=%s, err=%s" % (conn.host, cmd, ret, out, err))
            raise Exceptions.OperationError("Failed to get NFS share configuration on the node. host=%s, error=%s" % (conn.host, err), ret)

    return out.strip()

def get_mig_tasks(conn):
    ''' Get all migration task information with MAPI and return the result. '''
    if conn.host == 'localhost':
        method = HDIClient.HTTPMethod.GET
        path = "/mapi/MigrationTask"
        op = HDIOperation.RESTOperation(method, path)
    else:
        prog = os.path.join(BASEDIR, "int/get_configs")
        cmd = "sudo maintexec %s --mig-tasks" % prog
        op = HDIOperation.CommandOperation(cmd)

    log_info("Getting migration task information ...")
    try:
        ret, out, err = op.execute(conn)
    except HDIOperation.ConnectionError, e:
        log_error("Failed to communicate with %s. error=%s" % (conn.host, str(e)))
        raise Exceptions.ConnectionError(e.reason, e.code)

    if conn.host == 'localhost':
        if ret != 200:
            log_error("An unexpected HTTP status code. host=%s, path=%s, method=%s, ret=%s, out=%s, err=%s" % (conn.host, path, HDIClient.HTTPMethod.tostr(method), ret, out, err))
            raise Exceptions.OperationError("Failed to get migration task configuration on the node. host=%s, error=%s" % (conn.host, err), ret)
    else:
        if ret != 0:
            log_error("Command failed. host=%s, cmd=\"%s\", ret=%s, out=%s, err=%s" % (conn.host, cmd, ret, out, err))
            raise Exceptions.OperationError("Failed to get migration task configuration on the node. host=%s, error=%s" % (conn.host, err), ret)

    return out.strip()

def get_lft_setting(conn):
    ''' Get LFT settings with CLI and return the result. '''
    cmd = "sudo arclftctl --xml"
    op = HDIOperation.CommandOperation(cmd)

    log_info("Getting Large File Transfer settings ...")
    try:
        ret, out, err = op.execute(conn)
    except HDIOperation.ConnectionError, e:
        log_error("Failed to communicate with %s. error=%s" % (conn.host, str(e)))
        raise Exceptions.ConnectionError(e.reason, e.code)

    if ret != 0:
        log_error("Command failed. host=%s, cmd=\"%s\", ret=%s, out=%s, err=%s" % (conn.host, cmd, ret, out, err))
        raise Exceptions.OperationError("Failed to get Large File Transfer settings on the node. host=%s, error=%s" % (conn.host, err), ret)

    return out.strip()

def get_cache_resident_reporting(conn):
    ''' Get Cache Resdient reporting settings with CLI and return the result. '''
    cmd = "sudo arcresidentctl --xml"
    op = HDIOperation.CommandOperation(cmd)

    log_info("Getting Cache Resident reporting settings ...")
    try:
        ret, out, err = op.execute(conn)
    except HDIOperation.ConnectionError, e:
        log_error("Failed to communicate with %s. error=%s" % (conn.host, str(e)))
        raise Exceptions.ConnectionError(e.reason, e.code)

    if ret != 0:
        log_error("Command failed. host=%s, cmd=\"%s\", ret=%s, out=%s, err=%s" % (conn.host, cmd, ret, out, err))
        raise Exceptions.OperationError("Failed to get Cache Resident reporting settings on the node. host=%s, error=%s" % (conn.host, err), ret)

    return out.strip()

def get_network(conn):
    ''' Get all network interface information with MAPI and return the result. '''
    if conn.host == 'localhost':
        method = HDIClient.HTTPMethod.GET
        path = "/mapi/NetworkInterfaces"
        op = HDIOperation.RESTOperation(method, path)
    else:
        prog = os.path.join(BASEDIR, "int/get_configs")
        cmd = "sudo maintexec %s --network" % prog
        op = HDIOperation.CommandOperation(cmd)

    log_info("Getting network interface information ...")
    try:
        ret, out, err = op.execute(conn)
    except HDIOperation.ConnectionError, e:
        log_error("Failed to communicate with %s. error=%s" % (conn.host, str(e)))
        raise Exceptions.ConnectionError(e.reason, e.code)

    if conn.host == 'localhost':
        if ret != 200:
            log_error("An unexpected HTTP status code. host=%s, path=%s, method=%s, ret=%s, out=%s, err=%s" % (conn.host, path, HDIClient.HTTPMethod.tostr(method), ret, out, err))
            raise Exceptions.OperationError("Failed to get network configuration on the node. host=%s, error=%s" % (conn.host, err), ret)
    else:
        if ret != 0:
            log_error("Command failed. host=%s, cmd=\"%s\", ret=%s, out=%s, err=%s" % (conn.host, cmd, ret, out, err))
            raise Exceptions.OperationError("Failed to get network configuration on the node. host=%s, error=%s" % (conn.host, err), ret)

    return out.strip()

def get_hostname(conn):
    ''' Get hostname with CLI and return the result. '''
    # Directly get host name from an environment variable, not using MAPI.
    cmd = "echo ${HOSTNAME}"
    op = HDIOperation.CommandOperation(cmd)

    log_info("Getting hostname ...")
    try:
        ret, out, err = op.execute(conn)
    except HDIOperation.ConnectionError, e:
        log_error("Failed to communicate with %s. error=%s" % (conn.host, str(e)))
        raise Exceptions.ConnectionError(e.reason, e.code)

    if ret != 0:
        log_error("Command failed. host=%s, cmd=\"%s\", ret=%s, out=%s, err=%s" % (conn.host, cmd, ret, out, err))
        raise Exceptions.OperationError("Failed to get host name on the node. host=%s, error=%s" % (conn.host, err), ret)

    return out.strip()

def get_system_uuid(conn):
    ''' Get system UUID with CLI and return the result. '''
    # Directly get system UUID by reading a system file, not using MAPI.
    cmd = "curl -s -S file:///enas/hsmarc/conf/system_uuid.conf"
    op = HDIOperation.CommandOperation(cmd)

    log_info("Getting system UUID ...")
    try:
        ret, out, err = op.execute(conn)
    except HDIOperation.ConnectionError, e:
        log_error("Failed to communicate with %s. error=%s" % (conn.host, str(e)))
        raise Exceptions.ConnectionError(e.reason, e.code)

    if ret != 0:
        log_error("Command failed. host=%s, cmd=\"%s\", ret=%s, out=%s, err=%s" % (conn.host, cmd, ret, out, err))
        raise Exceptions.OperationError("Failed to get system UUID on the node. host=%s, error=%s" % (conn.host, err), ret)

    return out.strip()

def get_lu(conn):
    ''' Get all LU information with MAPI and return the result. '''
    if conn.host == 'localhost':
        method = HDIClient.HTTPMethod.GET
        path = "/mapi/LUs"
        op = HDIOperation.RESTOperation(method, path)
    else:
        prog = os.path.join(BASEDIR, "int/get_configs")
        cmd = "sudo maintexec %s --lu" % prog
        op = HDIOperation.CommandOperation(cmd)

    log_info("Getting LU information ...")
    try:
        ret, out, err = op.execute(conn)
    except HDIOperation.ConnectionError, e:
        log_error("Failed to communicate with %s. error=%s" % (conn.host, str(e)))
        raise Exceptions.ConnectionError(e.reason, e.code)

    if conn.host == 'localhost':
        if ret != 200:
            log_error("An unexpected HTTP status code. host=%s, path=%s, method=%s, ret=%s, out=%s, err=%s" % (conn.host, path, HDIClient.HTTPMethod.tostr(method), ret, out, err))
            raise Exceptions.OperationError("Failed to get LU configuration on the node. host=%s, error=%s" % (conn.host, err), ret)
    else:
        if ret != 0:
            log_error("Command failed. host=%s, cmd=\"%s\", ret=%s, out=%s, err=%s" % (conn.host, cmd, ret, out, err))
            raise Exceptions.OperationError("Failed to get LU configuration on the node. host=%s, error=%s" % (conn.host, err), ret)

    return out.strip()

def get_vg(conn):
    ''' Get all volume group information with MAPI and return the result. '''
    if conn.host == 'localhost':
        method = HDIClient.HTTPMethod.GET
        path = "/mapi/VolumeGroups"
        op = HDIOperation.RESTOperation(method, path)
    else:
        prog = os.path.join(BASEDIR, "int/get_configs")
        cmd = "sudo maintexec %s --vg" % prog
        op = HDIOperation.CommandOperation(cmd)

    log_info("Getting volume group information ...")
    try:
        ret, out, err = op.execute(conn)
    except HDIOperation.ConnectionError, e:
        log_error("Failed to communicate with %s. error=%s" % (conn.host, str(e)))
        raise Exceptions.ConnectionError(e.reason, e.code)

    if conn.host == 'localhost':
        if ret != 200:
            log_error("An unexpected HTTP status code. host=%s, path=%s, method=%s, ret=%s, out=%s, err=%s" % (conn.host, path, HDIClient.HTTPMethod.tostr(method), ret, out, err))
            raise Exceptions.OperationError("Failed to get Volume Group configuration on the node. host=%s, error=%s" % (conn.host, err), ret)
    else:
        if ret != 0:
            log_error("Command failed. host=%s, cmd=\"%s\", ret=%s, out=%s, err=%s" % (conn.host, cmd, ret, out, err))
            raise Exceptions.OperationError("Failed to get Volume Group configuration on the node. host=%s, error=%s" % (conn.host, err), ret)

    return out.strip()

def main():
    # Load configuration file
    config = Config.Config()

    # Parse options
    short_opts = 'h'
    long_opts = ['help', 'file-systems', 'history-count', 'cifs-shares', 'nfs-shares', 'mig-tasks', 'lft', 'cache-resident', 'network', 'hostname', 'system-uuid', 'lu', 'vg']
    try:
        opts, args = getopt.getopt(sys.argv[1:], short_opts, long_opts)
    except getopt.GetoptError, e:
        raise Exceptions.OptionError(e)

    if len(opts) != 1:
        raise Exceptions.OptionError("Only one option can be specified.")

    config_type = None
    fs = None
    for o, a in opts:
        if o in ('-h', '--help'):
            raise Exceptions.HelpOption()
        if o == '--file-systems':
            config_type = ConfigType.FILESYSTEM
        if o == '--history-count':
            config_type = ConfigType.HISTORY
            fs = a
        if o == '--cifs-shares':
            config_type = ConfigType.CIFSSHARE
        if o == '--nfs-shares':
            config_type = ConfigType.NFSSHARE
        if o == '--mig-tasks':
            config_type = ConfigType.MIGTASK
        if o == '--lft':
            config_type = ConfigType.LFT
        if o == '--cache-resident':
            config_type = ConfigType.RESIDENT
        if o == '--network':
            config_type = ConfigType.NETWORK
        if o == '--hostname':
            config_type = ConfigType.HOSTNAME
        if o == '--system-uuid':
            config_type = ConfigType.UUID
        if o == '--lu':
            config_type = ConfigType.LU
        if o == '--vg':
            config_type = ConfigType.VG

    if config_type is None:
        raise Exceptions.OptionError("No configuration type was specified.")

    # Setup logging
    log_file_path = os.path.join(config.log_dir, config.common_log_file)
    setup_rotation_logger(__name__, config.log_level, log_file_path, config.log_size, config.log_gen)

    # Build information required to connect
    # No need to specify SSH credential because we are going to make a connection to localhost.
    cred = HDIClient.Credential(None, config.rest_user, None, config.rest_passMD5)
    conn = HDIOperation.Connection("localhost", cred, config.retry_count, config.retry_interval, config.timeout)

    # Execute operation
    if config_type == ConfigType.FILESYSTEM:
        result = get_file_systems(conn)
    if config_type == ConfigType.HISTORY:
        result = get_history_count(conn, fs)
    elif config_type == ConfigType.CIFSSHARE:
        result = get_cifs_shares(conn)
    elif config_type == ConfigType.NFSSHARE:
        result = get_nfs_shares(conn)
    elif config_type == ConfigType.MIGTASK:
        result = get_mig_tasks(conn)
    elif config_type == ConfigType.LFT:
        result = str(get_lft_setting(conn))
    elif config_type == ConfigType.RESIDENT:
        result = get_cache_resident_reporting(conn)
    elif config_type == ConfigType.NETWORK:
        result = get_network(conn)
    elif config_type == ConfigType.HOSTNAME:
        result = get_hostname(conn)
    elif config_type == ConfigType.UUID:
        result = get_system_uuid(conn)
    elif config_type == ConfigType.LU:
        result = get_lu(conn)
    elif config_type == ConfigType.VG:
        result = get_vg(conn)

    print result

if __name__ == '__main__':
    try:
        main()
    except Exceptions.HelpOption:
        usage()
        sys.exit(0)
    except Exceptions.OptionError, e:
        log_error("Option error: %s" % str(e))
        sys.exit(1)
    except KeyboardInterrupt:
        print "Interrupted."
        log_warning("Interrupted.")
        sys.exit(2)
    except Config.BaseError, e:
        print "Unable to load script configuration: %s" % str(e)
        sys.exit(10)
    except Exceptions.ConnectionError, e:
        log_error("A connection error occurred: %s" % str(e))
        sys.exit(70)
    except Exceptions.OperationError, e:
        log_error("An operation error occurred: %s" % str(e))
        sys.exit(90)
    except EnvironmentError, e:
        log_exception("An internal error occurred: %s", e)
        sys.exit(99)
    except Exception, e:
        log_exception("An internal error occurred: %s", e)
        sys.exit(99)

    sys.exit(0)
