#!/usr/bin/python
from __future__ import with_statement

BASEDIR = "/home/nasroot/foscripts"
PROGRAM = "import_v2_settings"

import sys
import os
import getopt
import csv
import errno
import glob
import shutil

sys.path.append(os.path.join(BASEDIR, "bin/folib"))
import FailoverConfig
from CommonLogger import log_debug, log_info, log_warning, log_error, log_exception, setup_rotation_logger
import Utility
from Utility import print_status, print_status_error, print_status_warning
import HDIClient
import HDIOperation

sys.path.append(os.path.join(BASEDIR, "bin"))
import Config
import Exceptions
import getinfo_common
import setup_common
import setup_pair_group

HOMEDIR = "/home/nasroot"
V2DIR = os.path.join(BASEDIR, "_oldscripts")
V2FILES = (
    'api_password.sh',
    'backup_configs_alert.sh',
    'backup_configs.sh',
    'check_error.sh',
    'check_resources.sh',
    'collect_logs.sh',
    'delete_all_fs_node.sh',
    'delete_all_fs.sh',
    'display_dr_configuration.sh',
    'failover.sh',
    'flushmigration.sh',
    'fopylib',
    'get_backup_name.sh',
    'graceful_failback.sh',
    'graceful_failover.sh',
    'hdi_failover_lib',
    'hdi_failover_lib_function',
    'HdiHcpRest.pm',
    'HdiRest.pm',
    'int_check_api_auth.sh',
    'int_check_resources.sh',
    'int_check.sh',
    'int_cleanup.pl',
    'int_extract_fs_info.pl',
    'int_flushmigration.pl',
    'int_graceful_failover.sh',
    'int_init_lv.sh',
    'int_lft_tmp_del.py',
    'int_load_config_from_archive.sh',
    'int_restore_fs.sh',
    'int_restore_mig.sh',
    'int_restore_rehearsal.sh',
    'int_restore_residentpolicy.py',
    'int_restore.sh',
    'int_set_backup_name.sh',
    'int_show_result.sh',
    'int_stop_services.sh',
    'int_sync_fs.pl',
    'int_sync_pairs.sh',
    'int_update_api_password.sh',
    'lft_delete_tmp_objects.sh',
    'list_configs.sh',
    'register_cron.sh',
    'rehearsal_failover.sh',
    'restore_settings.sh',
    'save_settings.sh',
    'setup_basic_settings.sh',
    'sshloginOpenSSH.ppk',
    'sshloginOpenSSH.puk',
    'sync_fs_alert.sh',
    'sync_fs.sh',
    'sync_settings.sh',
    'tools',
    'restorecomp',
    'download',
    'prev_download',
    'error',
    'fobackup',
    'log',
    'hdi_dr_auth',
    'hdi_dr_backup',
    'hdi_dr_backup.example',
    'hdi_dr_backup.template',
    'hdi_dr_fssize',
    'hdi_dr_hosts',
    'hdi_dr_hosts.example',
    'hdi_dr_hosts.template',
    'hdi_dr_hosts.bak',
    'hdi_dr_hosts_drsite',
    'hdi_dr_model',
    'hdi_dr_self',
    'hdi_dr_self_backup',
    'hdi_dr_self.example',
    'hdi_dr_self.template',
    'hdi_dr_site',
    'hdi_dr_backup_*',
    '*.stderr',
    'int_graceful_failover.sh.stderr.*',
    'filesystem_detail*',
    'storage_list.xml',
    '*_cifs.xml',
    '*_nfs.xml',
    'errornum',
    'flag_synched'
    )

def usage():
    print "Usage:"
    print "    sudo maintexec %s [--complete]"
    print

def get_remote_fssize(node, config):
    # Assumes the remote node still have default SSH key...
    ssh_config = Config.SSHConfig(config.sys_conf_dir)
    cred = HDIClient.Credential(ssh_config.ssh_user, None, ssh_config.get_private_key(is_default=True), None)
    conn = HDIOperation.Connection(node, cred, config.retry_count, config.retry_interval, config.timeout)

    cmd = "curl file:///home/nasroot/hdi_dr_fssize"
    op = HDIOperation.CommandOperation(cmd)
    try:
        ret, out, err = op.execute(conn)
    except HDIOperation.ConnectionError, e:
        print_status_error("Unable to connect %s. error=%s" % (node, str(e)))
        raise Exceptions.ConnectionError(e.reason, e.code)

    if ret != 0:
        print_status_error("Unable to get fssize information on %s. ret=%s, out=%s, err=%s" % (node, ret, out, err))
        raise Exceptions.OperationError("Failed to read hdi_dr_fssize on %s." % node, ret)

    # out should be like 'fssize="64"'
    els = out.strip().split('"')
    if len(els) != 3:
        print_status_error("Wrong contents in hdi_dr_fssize. contents=%s" % out.strip())
        raise Exceptions.ConfigurationError("Wrong fssize definition.")

    return els[1]

def import_v2_settings(config):
    # Define a Configuration Group. Use an existing one if exists.
    try:
        confgroup_list = FailoverConfig.list_configuration_group(config.fo_conf_dir)
    except FailoverConfig.ConfigurationError, e:
        log_error("Failed to count number of configurations due to an configuration error. error=%s" % str(e))
        raise Exceptions.ConfigurationError(str(e))

    confgroup = setup_common.select_configuration_group(confgroup_list)

    # Setup a new failover configuration
    pairconf = FailoverConfig.PairConfig(config.fo_conf_dir, confgroup)
    stconf = FailoverConfig.StoragePolicyConfig(config.fo_conf_dir, confgroup)
    pgconf = FailoverConfig.PairGroupConfig(config.fo_conf_dir, confgroup)

    # Import pair definition
    print_status("Importing pair definition ...")
    pair_id_list = []
    hdi_dr_hosts = os.path.join(HOMEDIR, "hdi_dr_hosts")
    try:
        with open(hdi_dr_hosts, 'r') as f:
            reader = csv.reader(f)
            for items in reader:
                # Load information
                primary = items[0]
                secondary = items[1]
                primary_model = items[2]
                secondary_model = items[3]

                # Add a pair. Target file system is ALL
                pair_id = pairconf.add(primary, secondary, is_all=True)
                pair_id_list.append(pair_id)

                # Add a storage policy.
                fssize = get_remote_fssize(secondary, config)
                if fssize == 'same':
                    # Like for Like policy is defined
                    default_policy = FailoverConfig.StoragePolicy(FailoverConfig.StoragePolicy.FSDEF, policy=FailoverConfig.StoragePolicy.LIKEFORLIKE)
                elif secondary_model == 's':
                    # A fixed capacity for Auto with specified
                    default_policy = FailoverConfig.StoragePolicy(FailoverConfig.StoragePolicy.FSDEF, policy=FailoverConfig.StoragePolicy.AUTOSPEC, spec=int(fssize))
                elif secondary_model == 'c':
                    # Auto pickup one
                    default_policy = FailoverConfig.StoragePolicy(FailoverConfig.StoragePolicy.FSDEF, policy=FailoverConfig.StoragePolicy.AUTOPICKUP)

                stconf.add(pair_id, policy=default_policy)
    except EnvironmentError, e:
        print_status_error("An error occurred while reading %s. error=%s" % (hdi_dr_hosts, str(e)))
        raise

    # Create pair group
    print_status("Importing site definition ...")
    hdi_dr_site = os.path.join(HOMEDIR, "hdi_dr_site")
    try:
        with open(hdi_dr_site, 'r') as f:
            line = f.read()
    except EnvironmentError, e:
        print_status_error("Failed to read site information from %s. error=%s" % (hdi_dr_site, str(e)))
        raise

    site_line = line.strip()
    # site definition should be like 'site="Foo"'
    els = site_line.split('"')
    if len(els) != 3:
        print_status_error("Wrong contents in %s. contents=%s" % (hdi_dr_site, site_line))
        raise Exceptions.ConfigurationError("Wrong site definition.")

    site = els[1]
    pgconf.add(site, pair_id_list)

    # Confirmation
    print
    print "------ Imported settings ------"
    print
    print "====== Pair and storage policy configuration ======"
    setup_common.display_pairs(pairconf, stconf)
    print
    print "====== Pair group configuration ======"
    setup_pair_group.display_pair_groups(pgconf, pairconf)

    print
    answer = Utility.yes_or_no_full("Are you OK with the configuration above?")
    if answer is False:
        print_status("Process canceled.")
        raise Exceptions.NoExecution()

    # Save the configuration
    print_status("Saving the configuration ...")
    pairconf.save()
    stconf.save()
    pgconf.save()

    # Execute sync_settings
    print_status("Synchronizing the configuration with all nodes in the pair definition ...")
    prog = os.path.join(BASEDIR, "sync_settings")
    cmd = "sudo maintexec %s" % prog
    ret, out, err = Utility.run(cmd, display_stdout=True)
    if ret != 0:
        print_status_error("An error occurred while synchronizing configuration with other nodes.")
        # Rollback - remove all configuration
        print_status("Trying rollback ...")
        Utility.remove_file(pairconf.config_file)
        Utility.remove_file(pgconf.config_file)
        Utility.remove_file(stconf.config_file)
        raise Exceptions.OperationError("An error occurred on sync_settings.", ret)

    print_status("Import completed!")

    # Display guidance
    complete_prog = "%s --complete" % os.path.join(os.path.basename(BASEDIR), PROGRAM)
    register_cron_prog = os.path.join(os.path.basename(BASEDIR), "register_cron")
    print
    print "Next actions:"
    print "\t- Strongly recommend to execute check_resources and dry_run_failover to make sure the imported configuration is correct."
    print "\t- If you change any configuration (ex. add a pair, change storage policy, etc), you have to execute sync_settings to synchronize the updated settings to other nodes."
    print "\t- v2 scripts and configurations are still active. Once you confirmed the imported configuration is OK, please execute the following command."
    print "\t\t1. sudo maintexec %s" % complete_prog
    print "\t\t\tThis command will move all v2 scripts and configuration files into a directory and remove cron job of v2."
    print "\t\t2. sudo maintexec %s" % register_cron_prog
    print "\t\t\tThis command will register cron job of v3."
    print

def clean_up_v2_settings():
    # Create a directory to store v2 scripts and configuration files.
    print_status("Creating a directory to store v2 files ...")
    try:
        os.mkdir(V2DIR)
    except EnvironmentError, e:
        if e.errno == errno.EEXIST:
            log_info("The directory %s already exists." % V2DIR)
        else:
            print_status_error("Failed to create %s. error=%s" % (V2DIR, str(e)))
            raise

    # Move all v2 files to the directory.
    print_status("Moving v2 files to %s ..." % V2DIR)
    for entry in V2FILES:
        entry_path = os.path.join(HOMEDIR, entry)
        if '*' in entry_path:
            entries_to_be_moved = glob.glob(entry_path)
        else:
            entries_to_be_moved = [entry_path]
        for ent in entries_to_be_moved:
            from_entry = ent
            to_entry = os.path.join(V2DIR, os.path.basename(ent))
            log_debug("Moving %s to %s ..." % (from_entry, to_entry))
            try:
                shutil.move(from_entry, to_entry)
            except EnvironmentError, e:
                if e.errno == errno.ENOENT:
                    log_info("%s does not exist." % ent)
                else:
                    print_status_warning("Failed to move %s to %s. error=%s. Just skip it." % (from_entry, to_entry, str(e)))

    # Remove cron job of v2.
    print_status("Removing cron job of v2 ...")
    cmd = "sudo maintexec %s" % os.path.join(BASEDIR, "int/remove_v2_cron")
    ret, out, err = Utility.run(cmd, display_stdout=True)
    if ret != 0:
        print_status_error("An error occurred while removing cron job of v2. ret=%s" % ret)
        raise Exceptions.OperationError("Failed to remove cron job of v2.", ret)

    # Remove SSH key for v2
    print_status("Removing SSH key of v2 ...")
    comment = 'FOFB script key'
    cmd = "sudo sshkeydel -c '%s'" % comment
    ret, out, err = Utility.run(cmd, display_stdout=True)
    if ret != 0:
        print_status_error("An error occurred while removing SSH key of v2. ret=%s" % ret)
        raise Exceptions.OperationError("Failed to remove SSH key of v2.", ret)

    print_status("Clean up completed.")

def main():
    # Enable Ctrl-C interruption
    Utility.enable_interrupt()

    # Load configs
    config = Config.Config()

    # Setup logging
    log_file_path = os.path.join(config.log_dir, config.common_log_file)
    setup_rotation_logger(__name__, config.log_level, log_file_path, config.log_size, config.log_gen)

    log_info("----------------------- %s --------------------------" % PROGRAM)

    # Parse options
    short_opts = ''
    long_opts = ['complete']
    try:
        opts, args = getopt.getopt(sys.argv[1:], short_opts, long_opts)
    except getopt.GetoptError, e:
        raise Exceptions.OptionError(e)

    complete = False
    for o, a in opts:
        if o == '--complete':
            complete = True

    if not complete:
        import_v2_settings(config)
    else:
        clean_up_v2_settings()

if __name__ == '__main__':
    try:
        main()
    except Exceptions.NoExecution:
        sys.exit(0)
    except Exceptions.OptionError, e:
        log_error("Option error: %s" % str(e))
        usage()
        sys.exit(1)
    except KeyboardInterrupt:
        print "Interrupted."
        log_warning("Interrupted.")
        sys.exit(2)
    except Config.BaseError, e:
        print "Unable to load script configuration: %s" % str(e)
        sys.exit(10)
    except Exceptions.ConfigurationError, e:
        log_error("Unexpected configuration found: %s" % str(e))
        sys.exit(20)
    except Exceptions.ConnectionError, e:
        log_error("A connection error occurred: %s" % str(e))
        sys.exit(70)
    except Exceptions.OperationError, e:
        log_error("An operation error occurred: %s" % str(e))
        sys.exit(90)
    except EnvironmentError, e:
        log_exception("A system error occurred: %s", e)
        sys.exit(99)
    except Exception, e:
        log_exception("An internal error occurred: %s", e)
        sys.exit(99)

    sys.exit(0)
